package com.dayou.controller;

import com.dayou.dto.FinanceFineRefundSelectDTO;
import com.dayou.vo.FinanceFineRefundVO;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.springframework.web.bind.annotation.RestController;
import com.dayou.controller.BaseController;
import com.dayou.service.IFinanceFineRefundService;
import com.dayou.entity.FinanceFineRefund;
import com.dayou.common.RestResponse;
import org.springframework.web.bind.annotation.*;
import com.dayou.utils.ConvertUtil;
import com.dayou.utils.HttpKit;
import com.dayou.exception.ErrorCode;

import java.io.IOException;
import java.util.Date;
import java.util.List;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import org.springframework.http.MediaType;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;

import static com.dayou.enums.MainBusinessEnum.*;

/**
 * 罚款退款表
 *
 * @author GouGengquan
 * @since 2025-04-07
 */
@RestController
@RequestMapping("financeFineRefund")
@Slf4j
public class FinanceFineRefundController extends BaseController {

    @Autowired
    private IFinanceFineRefundService financeFineRefundService;

    /**
     * 条件分页查询罚款退款记录
     * @param page 分页
     * @param dto 查询条件
     * @return Page<FinanceFineRefundVO>
     */
    @GetMapping("/page")
    public RestResponse<Page<FinanceFineRefundVO>> page(Page page, FinanceFineRefundSelectDTO dto){
        Page<FinanceFineRefundVO> pages=financeFineRefundService.selectPage(page,dto);
        return RestResponse.data(pages);
    }

    /**
     * 导出罚款记录
     * @param dto 查询条件
     */
    @GetMapping("/export")
    public void exportList(FinanceFineRefundSelectDTO dto, HttpServletResponse response) throws IOException {
        List<FinanceFineRefundVO> financeFineRefundVOList = financeFineRefundService.exportList(dto);
        String businessName = null;
        if (dto.getBusinessType().equals(ASSET_BUSINESS.getCode())) {
            businessName = ASSET_BUSINESS.getMsg();
        } else if (dto.getBusinessType().equals(MAJOR_BUSINESS.getCode())) {
            businessName = MAJOR_BUSINESS.getMsg();
        } else {
            businessName = PERSONAL_BUSINESS.getMsg();
        }
        exportPlus(response,businessName + "_退款记录",financeFineRefundVOList, FinanceFineRefundVO.class);
    }

    /**
     * 罚款退款表详情
     */
    @GetMapping("/{id}")
    public RestResponse<FinanceFineRefund> detail(@PathVariable Long id){
        FinanceFineRefund xFinanceFineRefund =financeFineRefundService.detail(id);
        return RestResponse.data(xFinanceFineRefund);
     }

    /**
     * 罚款退款表新增
     */
    @PostMapping("")
    public RestResponse<Boolean> save(@RequestBody FinanceFineRefund financeFineRefund) {
        Boolean ret = financeFineRefundService.add(financeFineRefund);
        return RestResponse.data(ret);
    }

    /**
     * 更新罚款记录状态
     * @param financeFineRefund 更新内容
     * @return Boolean
     */
    @PutMapping("/updateStatus")
    public RestResponse<Boolean> updateStatus(@RequestBody FinanceFineRefund financeFineRefund) {
        Boolean ret = financeFineRefundService.updateStatus(financeFineRefund);
        return RestResponse.data(ret);
    }

    /**
     * 批量更新罚款记录状态
     * @param financeFineRefunds 更新内容
     * @return Boolean
     */
    @PutMapping("/updateStatus/batch")
    public RestResponse<Boolean> batchUpdateStatus(@RequestBody List<FinanceFineRefund> financeFineRefunds) {
        return RestResponse.data(financeFineRefundService.batchUpdateStatus(financeFineRefunds));
    }

    /**
     * 罚款退款表删除
     */
    @DeleteMapping("/{id}")
    public RestResponse<Boolean> delete(@PathVariable Long id) {
        Boolean ret = financeFineRefundService.delete(id);
        return RestResponse.data(ret);
    }


}

