package com.dayou.controller;

import com.dayou.dto.ProductionReturnSelectDTO;
import com.dayou.dto.WorkNodeCommit;
import com.dayou.vo.ProductionReturnVO;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.springframework.web.bind.annotation.RestController;
import com.dayou.controller.BaseController;
import com.dayou.service.IProductionReturnService;
import com.dayou.entity.ProductionReturn;
import com.dayou.common.RestResponse;
import org.springframework.web.bind.annotation.*;
import com.dayou.common.PullDownModel;
import com.dayou.utils.ConvertUtil;
import com.dayou.utils.HttpKit;
import com.dayou.exception.ErrorCode;

import java.io.IOException;
import java.util.Date;
import java.util.List;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import org.springframework.http.MediaType;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;

/**
 * 产品退回申请表
 *
 * @author GouGengquan
 * @since 2025-04-07
 */
@RestController
@RequestMapping("productionReturn")
@Slf4j
public class ProductionReturnController extends BaseController {
    @Autowired
    private IProductionReturnService productionReturnService;

    /**
     * 分页查询产品退回申请
     * @param page 分页
     * @param dto 查询条件
     * @return Page<ProductionReturnVO>
     */
    @GetMapping("/page")
    public RestResponse<Page<ProductionReturnVO>> page(Page page, ProductionReturnSelectDTO dto){
        Page<ProductionReturnVO> pages=productionReturnService.selectPage(page,dto);
        return RestResponse.data(pages);
    }

    /**
     * 导出产品退回申请
     * @param dto 查询条件
     */
    @GetMapping("/export")
    public void exportList(ProductionReturnSelectDTO dto, HttpServletResponse response) throws IOException {
        List<ProductionReturnVO> productionReturnVOList = productionReturnService.exportList(dto);
        exportPlus(response, "产品退回申请",productionReturnVOList, ProductionReturnVO.class);
    }

    /**
     * 产品退回申请表详情
     */
    @GetMapping("detail/{id}")
    public RestResponse<ProductionReturn> detail(@PathVariable Long id){
        ProductionReturn xProductionReturn =productionReturnService.detail(id);
        return RestResponse.data(xProductionReturn);
     }

    /**
     * 产品退回申请表新增
     */
    @PostMapping("/save")
    public RestResponse<Boolean> save(@RequestBody ProductionReturn productionReturn) {
        Boolean ret = productionReturnService.add(productionReturn);
        return RestResponse.data(ret);
    }

    /**
     * 产品退回申请表更新
     */
    @PutMapping("")
    public RestResponse<Boolean> update(@RequestBody ProductionReturn productionReturn) {
        Boolean ret = productionReturnService.update(productionReturn);
        return RestResponse.data(ret);
    }

    /**
     * 产品退回申请表删除
     */
    @DeleteMapping("/{id}")
    public RestResponse<Boolean> delete(@PathVariable Long id) {
        Boolean ret = productionReturnService.delete(id);
        return RestResponse.data(ret);
    }


    /**
     * 产品退回申请表下拉列表
     */
    @GetMapping("/simpleAll")
    public RestResponse<List<PullDownModel>> simpleAll(){
        LambdaQueryWrapper<ProductionReturn> lambdaQueryWrapper = new LambdaQueryWrapper<ProductionReturn>()
        .select(ProductionReturn::getId,ProductionReturn::getName)
        .eq(ProductionReturn::getDeleted,false);
        List<ProductionReturn> list= productionReturnService.list(lambdaQueryWrapper);
        List<PullDownModel> ret=ConvertUtil.copyList(list, PullDownModel.class);
        return RestResponse.data(ret);
    }
}

