package com.dayou.mapper;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.dayou.dto.FinanceFineSelectDTO;
import com.dayou.entity.FinanceFine;
import com.dayou.dao.CustomBaseMapper;
import com.dayou.entity.FinanceInvoice;
import com.dayou.vo.CompileFineAssetsOrder;
import com.dayou.vo.CompileFineMajorOrder;
import com.dayou.vo.FinanceFineVO;
import org.apache.ibatis.annotations.Param;

import java.math.BigDecimal;
import java.util.List;

/**
 * <p>
 * 超期未收款罚款表 Mapper 接口
 * </p>
 *
 * @author GouGengquan
 * @since 2025-04-07
 */
public interface FinanceFineMapper extends CustomBaseMapper<FinanceFine> {

    /**
     * 分页查询超期罚款
     * @param page 分页
     * @param financeFineSelectDTO 查询条件dto
     * @return Page<FinanceFineVO>
     */
    Page<FinanceFineVO> selectPage(@Param("page") Page page, @Param("dto") FinanceFineSelectDTO financeFineSelectDTO);

    /**
     * 导出查询超期罚款
     * @param financeFineSelectDTO 查询条件dto
     * @return List<FinanceFineVO>
     */
    List<FinanceFineVO> exportList(@Param("dto") FinanceFineSelectDTO financeFineSelectDTO);

    /**
     * 根据id确认罚款金额
     * @return Boolean
     */
    Boolean confirmFineAmount(@Param("id") Long id);

    /**
     * 根据id批量确认罚款金额
     * @return Boolean
     */
    Boolean batchConfirmFineAmount(@Param("ids") List<Long> ids);

    /**
     * 获取逾期资产产品款项信息(不包含预评)
     * @return List<CompileFineAssetsOrder>
     */
    List<CompileFineAssetsOrder> getAssetsOrder();

    /**
     * 获取逾期个贷产品款项信息(只查询结果报告)
     * @return List<Long>
     */
    List<Long> getPersonalOrder();

    /**
     * 获取逾期大中型产品款项信息
     * @return List<CompileFineAssetsOrder>
     */
    List<CompileFineMajorOrder> getMajorOrder();

    /**
     * 根据收款信息获取符合条件的实际开票金额的5%
     * @param orderFundId 订单收款id
     * @param productionFundId 产品收款id
     * @return BigDecimal 返回开票金额
     */
    BigDecimal getInvoiceByFundInfo(@Param("orderFundId") Long orderFundId, @Param("productionFundId") Long productionFundId);

    /**
     * 根据收款信息获取符合条件的罚款表信息
     * 且已确认罚款
     * @param orderFundId 订单收款id
     * @param productionFundId 产品收款id
     * @return FinanceFine
     */
    FinanceFine getFineByFundInfo(@Param("orderFundId") Long orderFundId, @Param("productionFundId") Long productionFundId);

}
