package com.dayou.entity;
import java.math.BigDecimal;

import com.baomidou.mybatisplus.annotation.TableField;
import com.dayou.annotation.*;
import com.dayou.common.BaseEntity;

import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.Date;

import lombok.Data;
import lombok.EqualsAndHashCode;

import static com.dayou.enums.MainBusinessEnum.FINANCE_MANAGEMENT;

/**
 * <p>
 * 财务实收款
 * </p>
 *
 * @author wucl
 * @since 2023-12-08
 */
@Data
@EqualsAndHashCode(callSuper = true)
@ExcelSheet(sheetName = "财务实收款")
@LuceneResource(value = FINANCE_MANAGEMENT,serviceImplClassName = "financeRealFundServiceImpl",menuName = "实收款管理",url = "/finance/real/fund")
public class FinanceRealFund extends BaseEntity {

    private static final long serialVersionUID=1L;

    /**
     * 付款方
     */
    @ImportCell
    @ExportCell(columnName = "付款方")
    @LuceneSearchable
    private String payer;

    /**
     * 付款银行
     */
    @ImportCell
    @ExportCell(columnName = "付款银行")
    @LuceneSearchable("payer_bank")
    private String payerBank;

    /**
     * 付款账号
     */
    @ImportCell
    @ExportCell(columnName = "付款账号")
    @LuceneSearchable("payer_account")
    private String payerAccount;

    /**
     * 付款金额
     */
    @ImportCell
    @ExportCell(columnName = "付款金额")
    @LuceneSearchable
    private BigDecimal amount;

    /**
     * 到账时日
     */
    @ImportCell
    @ExportCell(columnName = "到账时日")
    private LocalDate payDatetime;

    /**
     * 备注
     */
    @ImportCell
    @ExportCell(columnName = "备注")
    @LuceneSearchable
    private String remark;

    /**
     * 录入人
     */
    private Long creatorId;


    //到账时间范围
    @TableField(exist = false)
    private String startDate;
    @TableField(exist = false)
    private String endDate;

    //录入时间范围
    @TableField(exist = false)
    private String xStartDate;
    @TableField(exist = false)
    private String xEndDate;
    @TableField(exist = false)
    private Boolean notFinished;


}
