package com.dayou.entity;
import java.math.BigDecimal;

import com.baomidou.mybatisplus.annotation.TableField;
import com.dayou.common.BaseEntity;
import java.time.LocalDate;
import java.util.List;

import lombok.Data;
import lombok.EqualsAndHashCode;
import com.dayou.annotation.ExcelSheet;
import com.dayou.annotation.ExportCell;
import com.dayou.annotation.ImportCell;
/**
 * <p>
 * 大中型项目评估对象（土地 房地产类型）
 * </p>
 *
 * @author wucl
 * @since 2023-11-03
 */
@Data
@EqualsAndHashCode(callSuper = true)
@ExcelSheet(sheetName = "大中型项目评估对象")
public class MajorTarget extends BaseEntity {

    private static final long serialVersionUID=1L;

    /**
     * 项目id
     */
    private Long majorId;

    /**
     * 价值意见书号
     */
    @ImportCell
    @ExportCell(columnName = "价值意见书号")
    private String statementNo;

    /**
     * 报告号
     */
    @ImportCell
    @ExportCell(columnName = "报告号")
    private String reportNo;

    /**
     * 复评函号
     */
    @ImportCell
    @ExportCell(columnName = "复评函号")
    private String letterNo;

    /**
     * 估价对象类型（土地、房地产）
     */
    private String targetType;

    /**
     * 土地类型
     */
    @ImportCell
    @ExportCell(columnName = "土地类型")
    private String landType;

    /**
     * 土地使用权人
     */
    @ImportCell
    @ExportCell(columnName = "土地使用权人")
    private String rightToUser;

    /**
     * 国土证号 （土地使用证号）
     */
    @ImportCell
    @ExportCell(columnName = "国土证号")
    private String landCertificateNo;

    /**
     * 使用权面积
     */
    @ImportCell
    @ExportCell(columnName = "使用权面积")
    private BigDecimal useAcreage;

    /**
     * 坐落
     */
    @ImportCell
    @ExportCell(columnName = "坐落")
    private String located;

    /**
     * 剩余年限
     */
    @ImportCell
    @ExportCell(columnName = "剩余年限")
    private BigDecimal residueYears;

    /**
     * 容积率
     */
    @ImportCell
    @ExportCell(columnName = "容积率")
    private BigDecimal plotRatio;

    /**
     * (地类)用途
     */
    @ImportCell
    @ExportCell(columnName = "(地类)用途")
    private String landUse;

    /**
     * 使用权类型(出让，划拨)
     */
    @ImportCell
    @ExportCell(columnName = "使用权类型(出让，划拨)")
    private String rightType;

    /**
     * 项目负责人id
     */
    private Long principalId;

    /**
     * 实勘人
     */
    @ImportCell
    @ExportCell(columnName = "实勘人")
    private String siteIds;

    /**
     * 估价日期
     */
    @ImportCell
    @ExportCell(columnName = "估价日期")
    private LocalDate evaluateDate;

    /**
     * 备注
     */
    @ImportCell
    @ExportCell(columnName = "备注")
    private String remark;

    /**
     * 评估价格
     */
    @ImportCell
    @ExportCell(columnName = "评估价格")
    private BigDecimal evaluateValue;

    /**
     * 审核价格
     */
    private BigDecimal checkValue;

    /**
     * 估价方法
     */
    @ImportCell
    @ExportCell(columnName = "估价方法")
    private String evaluateMethods;

    /**
     * 是否审核
     */
    @ImportCell
    @ExportCell(columnName = "是否审核")
    private Boolean checked;

    /**
     * 打印份数
     */
    private Integer printing;

    /**
     * 打印备注
     */
    private String printingRemark;


    @TableField(exist = false)
    private List<Long> sites;

    // 房地产类型字段 包括但不限于
    /**
     * 房屋用途
     */
    private String housePurpose;
    /**
     *权利人
     */
    private String owner;
    /**
     *权证号
     */
    private String ownerNo;
    /**
     *项目名称
     */
    private String targetName;
    /**
     *竣工年代
     */
    private String buildDoneYear;
    /**
     *建筑面积
     */
    private BigDecimal buildAcreage;
    /**
     *土地面积
     */
    private BigDecimal landAcreage;
    /**
     *规划用途
     */
    private String  planPurpose;
    /**
     *实际用途
     */
    private String realPurpose;
    /**
     *实勘地址
     */
    private String siteLocated;

    /**
     * 使用权
     */
    private String useRightType;

    /**
     * 自定义其他评估方法
     */
    private String definedEvaluationMethod;


}
