package com.dayou.service;
import com.dayou.dto.FinanceFineRefundSelectDTO;
import com.dayou.entity.FinanceFineRefund;
import com.baomidou.mybatisplus.extension.service.IService;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.dayou.enums.FineRefundStatusEnum;
import com.dayou.vo.FinanceFineRefundVO;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.web.multipart.MultipartFile;

import java.util.List;

/**
 * <p>
 * 罚款退款表 服务类
 * </p>
 *
 * @author GouGengquan
 * @since 2025-04-07
 */
public interface IFinanceFineRefundService extends IService<FinanceFineRefund> {

        /**
         * 条件分页查询罚款退款记录
         * @param page 分页
         * @param dto 查询条件
         * @return Page<FinanceFineRefundVO>
         */
        Page<FinanceFineRefundVO> selectPage(Page page, FinanceFineRefundSelectDTO dto);

        /**
         * 导出罚款退款记录
         * @param dto 查询条件
         * @return List<FinanceFineRefundVO>
         */
        List<FinanceFineRefundVO> exportList(FinanceFineRefundSelectDTO dto);

        FinanceFineRefund detail(Long id);

        Boolean add(FinanceFineRefund financeFineRefund);

        /**
         * 新增退款记录,如果不存在
         * @param financeFineRefund 退款记录
         * @return Boolean
         */
        Boolean saveIfNotExists(FinanceFineRefund financeFineRefund);

        /**
         * 更新罚款记录状态
         * @param financeFineRefund 更新内容
         * @return Boolean
         */
        Boolean updateStatus(FinanceFineRefund financeFineRefund);

        /**
         * 批量更新罚款记录状态
         * @param financeFineRefunds 更新内容
         * @return Boolean
         */
        Boolean batchUpdateStatus(List<FinanceFineRefund> financeFineRefunds);

        Boolean delete(Long id);

}
