package com.dayou.service;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.dayou.dto.MajorStatisticalSelectDTO;
import com.dayou.vo.*;

import java.util.List;

public interface IMajorStatisticalStatementService {

    /**
     * 查询大中型台账
     * @param page 分页
     * @param dto 查询条件
     * @return Page<MajorLedgerVO>
     */
    Page<MajorLedgerVO> getMajorLedgerVO(Page page, MajorStatisticalSelectDTO dto);

    /**
     * 导出大中型台账
     * @param dto 查询条件
     * @return List<MajorLedgerVO>
     */
    List<MajorLedgerVO> exportMajorLedgerVO(MajorStatisticalSelectDTO dto);

    /**
     * 评估部绩效扣分查询
     * @param dto 查询dto
     * @return List<MajorPerformanceDeductionVO>
     */
    List<MajorPerformanceDeductionVO> getMajorPerformanceDeductionVO(MajorStatisticalSelectDTO dto);

    /**
     * 评估人员绩效扣分查询
     * @param dto 查询dto
     * @return List<MajorPerformanceDeductionVO>
     */
    List<MajorPerformanceDeductionVO> getMajorPerformanceDeductionVOByEvaluator(MajorStatisticalSelectDTO dto);

    /**
     * 根据部门id获取部门绩效扣分
     * @param dto 查询dto
     * @return List<MajorPerformanceDeductionVO>
     */
    List<MajorPerformanceDeductionVO> getMajorPerformanceDeductionVOByDepId(MajorStatisticalSelectDTO dto);

    /**
     * 获取每种错误类型的数量
     * @param dto 查询dto
     * @return List<MajorErrorTypeCountVO>
     */
    List<MajorErrorTypeCountVO> getMajorErrorTypeCountVO(MajorStatisticalSelectDTO dto);

    /**
     * 查询错误类型详情列表
     * @param page 分页
     * @param dto 查询dto
     * @return Page<MajorPerformanceDeductionDetailVO>
     */
    Page<MajorPerformanceDeductionDetailVO> getMajorErrorTypeDetailVO(Page page, MajorStatisticalSelectDTO dto);

    /**
     * 导出错误类型详情列表
     * @param dto 查询dto
     * @return List<MajorPerformanceDeductionDetailVO>
     */
    List<MajorPerformanceDeductionDetailVO> exportMajorErrorTypeDetailVO(MajorStatisticalSelectDTO dto);

    /**
     * 查询错误详情列表
     * @param page 分页
     * @param dto 查询dto
     * @return Page<MajorPerformanceDeductionDetailVO>
     */
    Page<MajorPerformanceDeductionDetailVO> getMajorPerformanceDeductionDetailVO(Page page, MajorStatisticalSelectDTO dto);

    /**
     * 导出错误详情列表
     * @param dto 查询dto
     * @return List<MajorPerformanceDeductionDetailVO>
     */
    List<MajorPerformanceDeductionDetailVO> exportMajorPerformanceDeductionDetailVO(MajorStatisticalSelectDTO dto);

    /**
     * 评估部效率查询
     * @param dto 查询dto
     * @return List<MajorEvaluateEfficiencyVO>
     */
    List<MajorEvaluateEfficiencyVO> getMajorEvaluateDepEfficiencyVO(MajorStatisticalSelectDTO dto);

    /**
     * 评估人员效率查询
     * @param dto 查询dto
     * @return List<MajorEvaluateEfficiencyVO>
     */
    List<MajorEvaluateEfficiencyVO> getMajorEvaluatePerEfficiencyVO(MajorStatisticalSelectDTO dto);

    /**
     * 效率详情列表查询
     * @param page 分页
     * @param dto 查询dto
     * @return Page<MajorEfficiencyDetailVO>
     */
    Page<MajorEfficiencyDetailVO> getMajorEfficiencyDetailVO(Page page, MajorStatisticalSelectDTO dto);

    /**
     * 效率详情列表导出
     * @param dto 查询dto
     * @return List<MajorEfficiencyDetailVO>
     */
    List<MajorEfficiencyDetailVO> exportMajorEfficiencyDetailVO(MajorStatisticalSelectDTO dto);

    /**
     * 项目部绩效查询
     * @param dto 查询dto
     * @return List<MajorEvaluateEfficiencyVO>
     */
    List<MajorDepPerformanceOverviewVO> getMajorDepPerformanceOverviewVO(MajorStatisticalSelectDTO dto);

    /**
     * 市场部效率查询
     * @param dto 查询dto
     * @return List<MajorMarketEfficiencyVO>
     */
    List<MajorMarketEfficiencyVO> getMajorsMarketDepEfficiencyVO(MajorStatisticalSelectDTO dto);

    /**
     * 客户经理效率查询
     * @param dto 查询dto
     * @return List<MajorMarketEfficiencyVO>
     */
    List<MajorMarketEfficiencyVO> getMajorMarketPerEfficiencyVO(MajorStatisticalSelectDTO dto);
}
