package com.dayou.service.impl;

import com.dayou.common.BaseEntity;
import com.dayou.dto.FinanceFineRefundSelectDTO;
import com.dayou.dto.SimplePostModel;
import com.dayou.entity.FinanceFineRefund;
import com.dayou.enums.FineRefundStatusEnum;
import com.dayou.mapper.FinanceFineRefundMapper;
import com.dayou.service.IFinanceFineRefundService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.dayou.utils.DateUtils;
import com.dayou.utils.LoginContext;
import com.dayou.vo.FinanceFineRefundVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.dayou.utils.ExcelUtil;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.CellStyle;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.web.multipart.MultipartFile;

import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.Date;
import java.util.List;
import java.util.ArrayList;
import org.springframework.transaction.annotation.Transactional;
import com.dayou.enums.BatchTaskTypeEnum;

/**
 * <p>
 * 罚款退款表 服务实现类
 * </p>
 *
 * @author GouGengquan
 * @since 2025-04-07
 */
@Service
public class FinanceFineRefundServiceImpl extends ServiceImpl<FinanceFineRefundMapper, FinanceFineRefund> implements IFinanceFineRefundService {

    @Autowired
    private FinanceFineRefundMapper financeFineRefundMapper;

    /**
     * 条件分页查询罚款退款记录
     * @param page 分页
     * @param dto 查询条件
     * @return Page<FinanceFineRefundVO>
     */
    @Override
    public Page<FinanceFineRefundVO> selectPage(Page page, FinanceFineRefundSelectDTO dto) {
        // 此处数据权限,客户经理只能查看自己的项目
        if (LoginContext.getLoginCacheUserBO().getPostList().stream()
                .map(SimplePostModel::getName)
                .anyMatch(name -> name.contains("客户经理") || name.contains("销售代表"))) {
            dto.setClientManagerId(LoginContext.getUserId());
        }
        return financeFineRefundMapper.selectPage(page, dto);
    }

    /**
     * 导出罚款退款记录
     * @param dto 查询条件
     * @return List<FinanceFineRefundVO>
     */
    @Override
    public List<FinanceFineRefundVO> exportList(FinanceFineRefundSelectDTO dto) {
        List<SimplePostModel> postModelList = LoginContext.getLoginCacheUserBO().getPostList();
        // 此处数据权限,客户经理只能查看自己的项目
        if (LoginContext.getLoginCacheUserBO().getPostList().stream()
                .map(SimplePostModel::getName)
                .anyMatch(name -> name.contains("客户经理") || name.contains("销售代表"))) {
            dto.setClientManagerId(LoginContext.getUserId());
        }
        return financeFineRefundMapper.exportList(dto);
    }


    @Override
    public FinanceFineRefund detail(Long id){
        return this.getById(id);
    }

    @Override
    public Boolean add(FinanceFineRefund financeFineRefund){
        return  this.save(financeFineRefund);
    }

    /**
     * 新增退款记录,如果不存在
     * @param financeFineRefund 退款记录
     * @return Boolean
     */
    @Override
    public Boolean saveIfNotExists(FinanceFineRefund financeFineRefund) {
        // 通过罚款id查询是否已经有存在的罚款后退款记录,避免重复生成退款记录
        boolean exists = this.count(new LambdaQueryWrapper<FinanceFineRefund>().eq(BaseEntity::getDeleted, false).eq(FinanceFineRefund::getFineId, financeFineRefund.getFineId())) > 0;
        if (!exists) {
            // 记录不存在，进行插入操作
            return this.save(financeFineRefund);
        }
        return false;
    }

    /**
     * 更新罚款记录状态
     * @param financeFineRefund 更新内容
     * @return Boolean
     */
    @Override
    public Boolean updateStatus(FinanceFineRefund financeFineRefund){
        financeFineRefund.setUserId(LoginContext.getUserId());
        financeFineRefund.setRefundDate(DateUtils.dateToLocalDateTime(new Date()));
        return this.updateById(financeFineRefund);
    }

    /**
     * 批量更新罚款记录状态
     * @param financeFineRefunds 更新内容
     * @return Boolean
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean batchUpdateStatus(List<FinanceFineRefund> financeFineRefunds) {
        Long userId = LoginContext.getUserId();
        LocalDateTime date = DateUtils.dateToLocalDateTime(new Date());
        for (FinanceFineRefund refund : financeFineRefunds) {
            refund.setUserId(userId);
            refund.setRefundDate(date);
        }
        return this.updateBatchById(financeFineRefunds);
    }

    @Override
    public Boolean delete(Long id){
        //逻辑删除
        return this.removeById(id);
    }
}
