package com.dayou.service.impl;

import com.baomidou.dynamic.datasource.annotation.DS;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.dayou.common.BaseEntity;
import com.dayou.entity.Assets;
import com.dayou.entity.Major;
import com.dayou.entity.OrderFund;
import com.dayou.entity.Personal;
import com.dayou.enums.MainBusinessEnum;
import com.dayou.mapper.OrderFundMapper;
import com.dayou.service.*;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;

import java.math.BigDecimal;
import java.util.Collections;
import java.util.List;

import static com.dayou.enums.MainBusinessEnum.ASSET_BUSINESS;

/**
 * <p>
 * 订单应收款 服务实现类
 * </p>
 *
 * @author wucl
 * @since 2023-12-14
 */
@DS("mbs")
@Service
public class OrderFundServiceImpl extends ServiceImpl<OrderFundMapper, OrderFund> implements IOrderFundService {

    @Autowired
    private OrderFundMapper orderFundMapper;

    @Autowired
    private IFinanceClaimService financeClaimService;

    @Autowired
    private IPersonalService personalService;

    @Autowired
    private IAssetsService assetsService;

    @Autowired
    private IMajorService majorService;

    @Override
    @SuppressWarnings("unchecked")
    public Page<OrderFund> selectPage(Page page,OrderFund orderFund){
        return this.page(page, new QueryWrapper<OrderFund>(orderFund));
    }


    @Override
    public OrderFund detail(Long id){
        return this.getById(id);
    }

    @Override
    public Boolean add(OrderFund orderFund){
        // 设置客户经理id
        switch (orderFund.getBusinessType()) {
            case "ASSET_BUSINESS":
                Assets assets = assetsService.getOne(new LambdaQueryWrapper<Assets>().eq(Assets::getOrderId, orderFund.getOrderId()).eq(Assets::getId,orderFund.getBusinessId()).eq(BaseEntity::getDeleted, Boolean.FALSE));
                if (assets != null) {
                    orderFund.setClientManagerId(assets.getClientManagerId());
                }
                break;
            case "PERSONAL_BUSINESS":
                Personal personal = personalService.getOne(new LambdaQueryWrapper<Personal>().eq(Personal::getOrderId, orderFund.getOrderId()).eq(Personal::getId,orderFund.getBusinessId()).eq(BaseEntity::getDeleted, Boolean.FALSE));
                if (personal != null) {
                    orderFund.setClientManagerId(personal.getClientManagerId());
                }
                break;
            case "MAJOR_BUSINESS":
                Major major = majorService.getOne(new LambdaQueryWrapper<Major>().eq(Major::getOrderId, orderFund.getOrderId()).eq(Major::getId,orderFund.getBusinessId()).eq(BaseEntity::getDeleted, Boolean.FALSE));
                if (major != null) {
                    orderFund.setClientManagerId(major.getClientManagerId());
                }
                break;
        }
        return  this.save(orderFund);
    }

    @Override
    public Boolean update(OrderFund orderFund){
        return  this.updateById(orderFund);
    }

    @Override
    public Boolean delete(Long id){
        //逻辑删除
        return this.removeById(id);
    }

    @Override
    public synchronized BigDecimal residueNotAllotAmount(Long orderFundId) {
        BigDecimal residue = orderFundMapper.residueNotAllotAmount(orderFundId);
        return residue;
    }

    @Override
    public void updateRealAmount(Long orderFundId) {
        BigDecimal totalAmount = financeClaimService.getTotalClaimAmountByOrderFundId(orderFundId);
        this.update(new LambdaUpdateWrapper<OrderFund>().set(OrderFund::getRealAmount,totalAmount)
                .eq(BaseEntity::getId,orderFundId));
    }

    @Override
    public void createOrderFundWhenOpeningOrder(Long id, String orderId, String name, MainBusinessEnum mainBusinessEnum, Long clientManagerId) {
        OrderFund orderFund = new OrderFund();
        orderFund.setBusinessType(mainBusinessEnum.name());
        orderFund.setOrderId(orderId);
        orderFund.setBusinessId(id);
        orderFund.setOrderName(name);
        orderFund.setClientManagerId(clientManagerId);
        this.save(orderFund);
    }

    @Override
    public List<OrderFund> getAllOrderFund() {
        return orderFundMapper.getAllOrderFund();
    }


}
