package com.dayou.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.dayou.common.BaseEntity;
import com.dayou.dto.ProductionReturnSelectDTO;
import com.dayou.dto.WorkNodeCommit;
import com.dayou.entity.FinanceFine;
import com.dayou.entity.OrderFund;
import com.dayou.entity.ProductionFund;
import com.dayou.entity.ProductionReturn;
import com.dayou.enums.workflow.NodeLogEnum;
import com.dayou.exception.BusinessException;
import com.dayou.exception.ErrorCode;
import com.dayou.mapper.FinanceInvoiceMapper;
import com.dayou.mapper.ProductionReturnMapper;
import com.dayou.service.IFinanceFineService;
import com.dayou.service.IOrderFundService;
import com.dayou.service.IProductionFundService;
import com.dayou.service.IProductionReturnService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.dayou.service.workflow.IWorkFlowService;
import com.dayou.utils.LoginContext;
import com.dayou.vo.ProductionReturnVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.dayou.utils.ExcelUtil;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.CellStyle;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.web.multipart.MultipartFile;
import java.util.List;
import java.util.ArrayList;
import org.springframework.transaction.annotation.Transactional;
import com.dayou.enums.BatchTaskTypeEnum;

import static com.dayou.enums.MainBusinessEnum.ASSET_BUSINESS;
import static com.dayou.enums.MainBusinessEnum.PRODUCTION_RETURN;
import static com.dayou.enums.ProductionReturnStatus.REVIEWING;

/**
 * <p>
 * 产品退回申请表 服务实现类
 * </p>
 *
 * @author GouGengquan
 * @since 2025-04-07
 */
@Service
public class ProductionReturnServiceImpl extends ServiceImpl<ProductionReturnMapper, ProductionReturn> implements IProductionReturnService {

    @Autowired
    private ProductionReturnMapper productionReturnMapper;

    @Autowired
    private IProductionFundService productionFundService;

    @Autowired
    private IOrderFundService orderFundService;

    @Autowired
    private FinanceInvoiceMapper financeInvoiceMapper;

    @Autowired
    private IWorkFlowService workFlowService;

    /**
     * 分页查询产品退回申请
     * @param page 分页
     * @param dto 查询条件
     * @return Page<ProductionReturnVO>
     */
    @Override
    public Page<ProductionReturnVO> selectPage(Page page, ProductionReturnSelectDTO dto){
        // 判断是否是查询自己的待办
        if (dto.getQueryScope().equals("TODO")) {
            dto.setUserId(LoginContext.getUserId());
        }
        Page<ProductionReturnVO> productionReturnVOPage = productionReturnMapper.selectPage(page, dto);
        List<ProductionReturnVO> records = productionReturnVOPage.getRecords();
        for (ProductionReturnVO vo : records) {
            switch (vo.getBusinessType()) {
                case "ASSET_BUSINESS":
                    switch (vo.getProductionType()) {
                        case "LETTER":
                            vo.setProductionType("意见函");
                            break;
                        case "CONSULT":
                            vo.setProductionType("咨询报告");
                            break;
                        case "REPORT":
                            vo.setProductionType("评估报告");
                            break;
                        default:
                            vo.setProductionType("-");
                            break;
                    }
                    break;
                case "PERSONAL_BUSINESS":
                    switch (vo.getProductionType()) {
                        case "FINAL":
                            vo.setProductionType("结果报告");
                            break;
                        case "TECHNIC":
                            vo.setProductionType("技术报告");
                            break;
                        default:
                            vo.setProductionType("-");
                            break;
                    }
                    break;
                case "MAJOR_BUSINESS":
                    switch (vo.getProductionType()) {
                        case "LETTER":
                            vo.setProductionType("复评函");
                            break;
                        case "STATEMENT":
                            vo.setProductionType("价值意见书");
                            break;
                        case "REPORT":
                            vo.setProductionType("报告");
                            break;
                        default:
                            vo.setProductionType("-");
                            break;
                    }
                    break;
            }
        }
        return productionReturnVOPage;
    }

    /**
     * 导出产品退回申请
     * @param dto 查询条件
     * @return Page<ProductionReturnVO>
     */
    @Override
    public List<ProductionReturnVO> exportList(ProductionReturnSelectDTO dto) {
        // 判断是否是查询自己的待办
        if (dto.getQueryScope().equals("TODO")) {
            dto.setUserId(LoginContext.getUserId());
        }
        List<ProductionReturnVO> productionReturnVOList = productionReturnMapper.exportList(dto);
        for (ProductionReturnVO vo : productionReturnVOList) {
            switch (vo.getBusinessType()) {
                case "ASSET_BUSINESS":
                    switch (vo.getProductionType()) {
                        case "LETTER":
                            vo.setProductionType("意见函");
                            break;
                        case "CONSULT":
                            vo.setProductionType("咨询报告");
                            break;
                        case "REPORT":
                            vo.setProductionType("评估报告");
                            break;
                        default:
                            vo.setProductionType("-");
                            break;
                    }
                    break;
                case "PERSONAL_BUSINESS":
                    switch (vo.getProductionType()) {
                        case "FINAL":
                            vo.setProductionType("结果报告");
                            break;
                        case "TECHNIC":
                            vo.setProductionType("技术报告");
                            break;
                        default:
                            vo.setProductionType("-");
                            break;
                    }
                    break;
                case "MAJOR_BUSINESS":
                    switch (vo.getProductionType()) {
                        case "LETTER":
                            vo.setProductionType("复评函");
                            break;
                        case "STATEMENT":
                            vo.setProductionType("价值意见书");
                            break;
                        case "REPORT":
                            vo.setProductionType("报告");
                            break;
                        default:
                            vo.setProductionType("-");
                            break;
                    }
                    break;
            }
        }
        return productionReturnVOList;
    }


    @Override
    public ProductionReturn detail(Long id){
        return this.getById(id);
    }

    /**
     * 发起产品退回流程
     * @param productionReturn 产品退回信息
     * @return WorkNodeCommit
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean add(ProductionReturn productionReturn){
        Long orderFundId = null;
        Long productionFundId = null;
        // 获取收款信息
        OrderFund orderFund = orderFundService.getOne(new LambdaQueryWrapper<OrderFund>()
                .eq(BaseEntity::getDeleted, false)
                .eq(OrderFund::getBusinessType, productionReturn.getBusinessType())
                .eq(OrderFund::getBusinessId, productionReturn.getBusinessId()));
        if(ObjectUtil.isNotNull(orderFund)) { // 判断有没有订单收款信息
            ProductionFund productionFund = productionFundService.getOne(new LambdaQueryWrapper<ProductionFund>()
                    .eq(BaseEntity::getDeleted, false)
                    .eq(ProductionFund::getBusinessType, productionReturn.getBusinessType())
                    .eq(ProductionFund::getBusinessId, productionReturn.getBusinessId())
                    .eq(ProductionFund::getProductionNo, productionReturn.getProductionNo())
                    .eq(ProductionFund::getProductionType, productionReturn.getProductionType())
                    .eq(ProductionFund::getOrderFundId, orderFund.getId()));
            orderFundId = orderFund.getId();
            productionFundId = productionFund != null ? productionFund.getId() : null;
        }
        // 获取发票号
        productionReturn.setInvoiceNo(financeInvoiceMapper.getInvoiceNoByFundId(orderFundId, productionFundId));
        Integer count = productionReturnMapper.count(productionReturn);
        if (count > 0) {
            ErrorCode.throwBusinessException(ErrorCode.CUSTOM_ERROR, "当前已有审核中或已通过的流程,无需重复提交");
        }
        productionReturn.setClinetManagerId(LoginContext.getCurrentUserId());
        productionReturn.setStatus(REVIEWING.getCode());
        this.save(productionReturn);
        // 发起退回流程
        return workFlowService.openingProductionReturn(PRODUCTION_RETURN, productionReturn.getId());
    }

    @Override
    public Boolean update(ProductionReturn productionReturn){
        return  this.updateById(productionReturn);
    }

    @Override
    public Boolean delete(Long id){
        //逻辑删除
        return this.removeById(id);
    }
}
